# -*- coding: utf-8 -*-
"""
Pydantic schemas for TOAR database

"""

from typing import List
from pydantic import BaseModel, Field, BaseConfig, Json
from geoalchemy2 import WKTElement
from geoalchemy2.shape import to_shape
import datetime as dt


# the following class was taken from:
# https://github.com/tiangolo/fastapi/issues/312
class Coordinates(BaseModel):
    lat: float = Field(0, gte=-90, lte=90)
    lng: float = Field(0, gte=-180, lte=180)
    alt: float

# ======== StationmetaCore =========

class StationmetaCoreBase(BaseModel):
    id: int = None
    codes: List[str] = []
    name: str
    coordinates: Coordinates
    country: str
    state: str
    coordinate_validation_status: int
    coordinate_validation_date: dt.datetime
    type_of_environment: int
    type_of_area: int
    category: str
    timezone: str
    additional_metadata: Json
    coordinate_validator_id: int

    class Config(BaseConfig):
        arbitrary_types_allowed = True


class StationmetaCoreCreate(StationmetaCoreBase):
    pass


class StationmetaCore(StationmetaCoreBase):
    id: int

    class Config:
        orm_mode = True


def get_geom_from_coordinates(coordinates: Coordinates) -> str:
    geom_wkte = f'SRID=4326;POINTZ({coordinates.lng} {coordinates.lat} {coordinates.alt})'
    return geom_wkte


def get_coordinates_from_geom(geom: WKTElement) -> Coordinates:
    shply_geom = to_shape(geom)
    coordinates = Coordinates(lng=shply_geom.x, lat=shply_geom.y, alt=shply_geom.z)
    return coordinates

# ======== StationmetaAnnotation =========

class StationmetaAnnotationBase(BaseModel):
    id: int = None
    kind: int
    text: str
    date_added: dt.datetime
    approved: bool
    contributor_id: int


class StationmetaAnnotationCreate(StationmetaAnnotationBase):
    pass


class StationmetaAnnotation(StationmetaAnnotationBase):
    id: int

    class Config:
        orm_mode = True

# ======== StationmetaAux =========

# -------- StationmetaAuxDoc ---------

class StationmetaAuxDocBase(BaseModel):
    id: int = None
    resource_description: str
    date_added: dt.datetime
    resource: str
    station_id: int


class StationmetaAuxDocCreate(StationmetaAuxDocBase):
    pass


class StationmetaAuxDoc(StationmetaAuxDocBase):
    id: int

    class Config:
        orm_mode = True

# -------- StationmetaAuxImage ---------

class StationmetaAuxImageBase(BaseModel):
    id: int = None
    resource_description: str
    date_added: dt.datetime
    resource: str
    image_height: int
    image_width: int
    station_id: int


class StationmetaAuxImageCreate(StationmetaAuxImageBase):
    pass


class StationmetaAuxImage(StationmetaAuxImageBase):
    id: int

    class Config:
        orm_mode = True

# -------- StationmetaAuxUrl ---------

class StationmetaAuxUrlBase(BaseModel):
    id: int = None
    resource_description: str
    date_added: dt.datetime
    resource: str
    station_id: int

class StationmetaAuxUrlCreate(StationmetaAuxUrlBase):
    pass


class StationmetaAuxUrl(StationmetaAuxUrlBase):
    id: int

    class Config:
        orm_mode = True

# ======== StationmetaGlobal =========

class StationmetaGlobalBase(BaseModel):
    id: int = None
    population_density_year2010: float
    max_population_density_25km_year2010: float
    climatic_zone: str
    nightlight_1km_year2013: float
    nightlight_5km_year2013: float
    max_nightlight_25km_year2013: float
    wheat_production_year2000: float
    rice_production_year2000: float
    edgar_htap_v2_nox_emissions_year2010: float
    omi_no2_column_years2011to2015: float
    htap_region_tier1: int
    etopo_alt: float
    etopo_min_alt_5km: float 
    etopo_relative_alt: float
    dominant_landcover_year2012: int
    toar1_category: int
    station_id: int


class StationmetaGlobalCreate(StationmetaGlobalBase):
    pass


class StationmetaGlobal(StationmetaGlobalBase):
    id: int

    class Config:
        orm_mode = True


class StationmetaGlobalBaseNested(BaseModel):
    id: int = None
    population_density_year2010: float = None
    max_population_density_25km_year2010: float = None
    climatic_zone: str = None
    nightlight_1km_year2013: float = None
    nightlight_5km_year2013: float = None
    max_nightlight_25km_year2013: float = None
    wheat_production_year2000: float = None
    rice_production_year2000: float = None
    edgar_htap_v2_nox_emissions_year2010: float = None
    omi_no2_column_years2011to2015: float = None
    htap_region_tier1: int = None
    etopo_alt: float = None
    etopo_min_alt_5km: float  = None
    etopo_relative_alt: float = None
    dominant_landcover_year2012: int = None
    toar1_category: int = None


class StationmetaGlobalNestedCreate(StationmetaGlobalBaseNested):
    pass


class StationmetaGlobalNested(StationmetaGlobalBaseNested):
    id: int

    class Config:
        orm_mode = True


# ======== StationmetaGlobalService =========

class StationmetaGlobalServiceBase(BaseModel):
    id: int = None
    variable_name: str
    result_type: int
    result_nvalues: int
    service_valid_year: int
    service_url: str


class StationmetaGlobalServiceCreate(StationmetaGlobalServiceBase):
    pass


class StationmetaGlobalService(StationmetaGlobalServiceBase):
    id: int

    class Config:
        orm_mode = True

# ======== StationmetaRole =========

class StationmetaRoleBase(BaseModel):
    id: int = None
    role: int
    status: int
    person_id: int


class StationmetaRoleCreate(StationmetaRoleBase):
    pass


class StationmetaRole(StationmetaRoleBase):
    id: int

    class Config:
        orm_mode = True

# ======== for nested view/upload =========

class StationmetaBase(StationmetaCoreBase):
    roles: List[StationmetaRole] = None
    annotations: List[StationmetaAnnotation] = None
    aux_images: List[StationmetaAuxImage] = None
    aux_docs: List[StationmetaAuxDoc] = None
    aux_urls: List[StationmetaAuxUrl] = None
    globalmeta: StationmetaGlobal = None
    globalservice: StationmetaGlobalService = None

    class Config:
       orm_mode = True

class StationmetaCreate(StationmetaCoreBase):
    roles: List[StationmetaRoleBase] = None
    annotations: List[StationmetaAnnotation] = None
    aux_images: List[StationmetaAuxImage] = None
    aux_docs: List[StationmetaAuxDoc] = None
    aux_urls: List[StationmetaAuxUrl] = None
    globalmeta: StationmetaGlobalNestedCreate = None
    globalservice: StationmetaGlobalService = None

    class Config:
        orm_mode = True


class Stationmeta(StationmetaBase):
    id: int

    class Config:
      orm_mode = True

